% mkcomposite4s.m Script for making the LR composite results analyzed
% in McGregor et al. (2015)
% 
% MNE 3 August 2015 (mnevans@umd.edu) 
%
% Outline of compositing code steps:
%
% 0. Data source information.
% 1. Read in the input dataset.
% 2. Create bin composites.
% 3. Plot results shown in Supplementary Fig 1a and Fig 2a of paper.
% 4. Save the work.
%
% Code tested in:
% (a) matlab R2012b in Debian/Jessie. Note: Code requires
% the matlab statistics toolbox for functions: nansum, nanmedian, nanmean.
%
% (b) octave 3.8.2 for x86_64-pc-linux-gnu (freeware available at:
% www.octave.org) in Debian/Jessie.  Note: comment out line 299
% ("boxplot...") for the code to complete.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%
% 0. Data sourcing information:
%
% Citation: McGregor, H.V., Evans, M.N., Goosse, H., Leduc, G., Martrat, B.,
% Addison, J., Mortyn, P.G., Oppo, D.W., Seidenkrantz, M.-S., Sicre,
% M.-A., Phipps, S.J., Selvaraj, K., Thirumalai, K., Filipsson, H.L.,
% Ersek, V. (2015) Robust global ocean cooling trend for the
% pre-industrial Common Era.DOI: 10.1038/NGEO2510. Nature Geoscience.
% 
% Input data is here:
%
% Ocean2kLR2015sst.csv
%
% Sourcing information is in the Excel workbook file:
% 
% Ocean2kLR2015sst.xlsx, sheet "Datamatrix"
%
% with metadata, footnotes and URLS for original data sources in
% accompanying sheets.
%
% This is itself an extract, by the Ocean2k LR Working Group, from the
% Ocean2k paleodata metadatabase:
% http://pages-igbp.org/ini/wg/ocean2k/intro
%
% Last revised February 20, 2015.
%
% The data matrix format (sheet: Datamatrix) is: 

% [chron1 dataseries1 chron2 dataseries2 ... chronN dataseriesN] so
% there are twice as many columns as dataseries, and each pair of
% columns corresponds to the unique chronology and dataseries for each
% input dataset.
% 
% Data are all organized by time, time running from most recent to
% most ancient, first row to last row.  The input data is made into a
% rectangular matrix by writing NaN (Not A Number) into all blank
% spreadsheet cells.

% End data origin description
% ===========================

clear;close all

% 1. Read in the input dataset

load input_data/Ocean2kLR2015sst.csv
input=Ocean2kLR2015sst;

[nt nd]=size(input); % get the dimensions of the input dataset.
time=input(:,[1:2:nd-1]); % get the time data: first,
                          % third,.. second to last columns
dat=input(:,[2:2:nd]); % get the corresponding SST reconstructions:
                       % second, fourth, ... last columns.

% 2.  Create bin composites:

% 2.1. Populate each bin with data that date to within that bin:

% Binning parameters: set resolution, start year, end year for
% compositing: if the increment is 200y, composite from 0-2000 at 200y
% increment ("bins");

incr = 200; % 200 year compositing window
sy=2000; % most recent year for compositing (most recent bin edge)
ey=0; % most ancient year for compositing (most ancient bin edge)

% set the range for bin compositing.
range=[sy:-incr:ey];
% array factor: arbitrary allocation for matrix space for later compositing
nb=50; %  nb is used here to make extra
% rows to keep data in the bins.
 % create initial composite matrix, allowing for nd*nb numbers per bin:
binn=NaN.*zeros(length(range)-1,nd*nb);  
% another matrix of like size is created for storing composited time data:
stime=binn; 

% 2.2 Create Addison means: 
%
% for all data with more than one point within the bin, average those
% values to one point within the bin, so that there are the same
% number of averages within each bin as there are number of datasets
% having values within the bin.  Then standardize the resulting time
% series averages to their resolution time mean and standard
% deviation.  The idea is to avoid weighting the bin toward those
% datasets that have high resolution and therefore many points in a
% given bin, relative to others.

a=0; % set a counter to zero
% create initial composite matrix, allowing for as many numbers per
% bin as there are potential datasets in each bin:
binnm=NaN.*zeros(length(range)-1,size(dat,2)); 
 % another matrix of like size as bin for storing time data:
stimem=binnm;
% This matrix will be used to store the number of points used for each
% Addison mean:
binnNm=binnm; 

format bank
% Loop over bins:
for y=1:length(range)-1 % loop through the bins
  rangec(y)=mean([range(y) range(y+1)+1]);
  [range(y) floor(rangec(y)) range(y+1)+1] % output the bin edges and centers
  a=a+1; % increment the counter into bin row
  
  % now loop over the chronology for each dataseries individually:
  for z=1:size(time,2)
      % find the positions in the time matrix for each dataseries z with
      % data collected at ages that fall in the bin (given by variable
      % y):
      if y~=length(range),
          i=find(time(:,z) <= range(y) & time(:,z) > (range(y+1)+ ...
                                                      1));
      end
      % now write the corresponding data into bin row a:
      if ~isempty(i) & y~=length(range),
          % if there are data that fall in the bin, and we're not
          % at the edge of the first bin (0 CE),
          % then average them into a single value for that bin (row) and that
          % dataseries' column:
          % here are averaged times (can be used to check algorithm
          % and representation with respect to time in the bin):
          stimem(a,z)=mean(time(i,z));
          % mean of the data (dat) within the bin:
          binnm(a,z)=mean(dat(i,z)); 
          % save a record of the number of data points going into
          % the series' Addison average:
          binnNm(a,z)=length(i);
      end
  end
  % If there are any Addison means in the bin, count the number of
  % Addison means per bin:
  if ~isempty(length(find(~isnan(binnm(y,:))))),
      nval(y)=length(find(~isnan(binnm(y,:))));
  else
      nval(y)=0; % there are no Addison means in the bin
  end
end
% For incr = 200y, sy=2000, ey=0, there are now 10 bins:
% 2000...1801,
% 1800...1601,
% 1600...1401,
% 1400...1201,
% 1200...1001,
% 1000...801,
% 800...601,
% 600...401,
% 400...201,
% 200...1

% 2.3. Standardize after Addison binning for further analysis:
% create a new matrix for standardized data, same dimensions as binnm:
binnm_nm=binnm; 
% loop over the bins (rows)
% create a copy of the standardization subset: 

nc1=1; % standardization: more recent bin to:
nc2=10; % standardization: most distant bin.
binnm_nmc=binnm(nc1:nc2,:);

for i=1:size(binnm_nm,2);
    % find the non-missing rows for each data series (columns)
    % within the standardization interval:
    j=find(~isnan(binnm_nmc(:,i)));
    % how many points used for standardization?
    length_nmc(i)=length(j);
    mn(i)=mean(binnm_nmc(j,i));
    st(i)=std(binnm_nmc(j,i));
    % calculate the Addison value = standardized value, standardized on
    % means and standard deviations of bin-meaned data:
    k=find(~isnan(binnm_nm(:,i)));
    binnm_nm(k,i)=(binnm(k,i)-mn(i))./(st(i));
end
% Check: do all of the dataseries have valid standardizations?  no;
% for global set, 45 and 46 do not: they only have data in the very
% last bin (2000...1801), so their standard deviations are undefined.
% Set their standardized values to NaN:
binnm_nm(isinf(binnm_nm))=NaN;

% 3. Plot the results that are in Fig 2a and Supplementary Figure 1a.

% 3.1. Supplementary Figure 1a: Plot the input dataseries all on top of one
% another, colorized by basin location

pacd=38:53;% Pacific      : series 38-53
arcd=1:2;  % Arctic       : series  1,2
atld=3:31; % Atlantic     : series  3-31 
socd=54:57;% Southern     : series 54-57
indd=32:33;% Indian       : series 32,33
medd=34:37;% Mediterranean: series 34-37

% first plot one series from each site for the legend:
% Pacific:
subplot(211);plot(time(:,pacd(1)),dat(:,pacd(1)),'Color',[241 137 39]./255)
set(gca,'Fontname','Arial');
get(gca,'Fontname');

hold on
plot(time(:,socd(1)),dat(:,socd(1)),'Color',[147 91 161]./255) % Southern
set(gca,'Fontname','Arial')
plot(time(:,atld(1)),dat(:,atld(1)),'color',[36 170 226]./255) % Atlantic
plot(time(:,medd(1)),dat(:,medd(1)),'color',[9 129 54]./255) % Med
plot(time(:,arcd(1)),dat(:,arcd(1)),'color',[221 11 21]./255) % Arc
plot(time(:,indd(1)),dat(:,indd(1)),'color',[201 110 170]./255) % Ind
legend(['PAC(' num2str(length(pacd)) ')'],['SOU(' ...
                    num2str(length(socd)) ')'],['ATL(' ...
                    num2str(length(atld)) ')'],['MED(' num2str(length(medd)) ...
                    ')'],['ARC(' num2str(length(arcd)) ')'],['IND(' ...
                    num2str(length(indd)) ')'],'Location','EastOutside')
% now plot all the data in the same legend colors
plot(time(:,pacd),dat(:,pacd),'Color',[241 137 39]./255)% Pac
ylabel('Reconstructed SST (^{o}C)')
xlabel('Year (CE)')
plot(time(:,socd),dat(:,socd),'Color',[147 91 161]./255) % Southern
plot(time(:,atld),dat(:,atld),'color',[36 170 226]./255) % Atlantic
plot(time(:,medd),dat(:,medd),'color',[9 129 54]./255) % Med
plot(time(:,arcd),dat(:,arcd),'color',[221 11 21]./255) % Arc
plot(time(:,indd),dat(:,indd),'color',[201 110 170]./255) % Ind
hold off
set(gca,'Fontname','Arial')
axis([0 2000 -2 30]);
set(gca,'Fontname','Arial')
set(gca,'XTick',[0:200:2000])
set(gca,'Fontname','Arial')
title(['                               McGregor et al (2015), Supplementary Fig. 1a: Input SST ' ...
       'reconstructions'])    

grid on


% 3.2. Plot the Addison-mean, standarized data series shown in Fig 2a.

% 3.2.1. Area weighting: see Supplementary Information for details and
% references
pacw=0.384;
atlw=0.183;
medw=0.008;
socw=0.278;
arcw=0.034;
indw=0.113;
tw=sum([pacw atlw medw socw arcw indw ]);

% area weight is mean(dat) * weight(region)
pacdwmn=pacw.*nanmean(binnm_nm(:,pacd),2);
arcdwmn=arcw.*nanmean(binnm_nm(:,arcd),2);
atldwmn=atlw.*nanmean(binnm_nm(:,atld),2);
socdwmn=socw.*nanmean(binnm_nm(:,socd),2);
inddwmn=indw.*nanmean(binnm_nm(:,indd),2);
meddwmn=medw.*nanmean(binnm_nm(:,medd),2);
wavemn=nansum([pacdwmn arcdwmn atldwmn socdwmn inddwmn meddwmn],2);

% area weight is median(dat)*weight(region)
pacdwmd=pacw.*nanmedian(binnm_nm(:,pacd),2);
arcdwmd=arcw.*nanmedian(binnm_nm(:,arcd),2);
atldwmd=atlw.*nanmedian(binnm_nm(:,atld),2);
socdwmd=socw.*nanmedian(binnm_nm(:,socd),2);
inddwmd=indw.*nanmedian(binnm_nm(:,indd),2);
meddwmd=medw.*nanmedian(binnm_nm(:,medd),2);
wavemd=nansum([pacdwmd arcdwmd atldwmd socdwmd inddwmd meddwmd],2);


subplot(212)
plot(floor(rangec),binnm_nm)
plot(10:-1:1,binnm_nm(:,pacd),'Color',[241 137 39]./255) % Pacific
hold on
plot(10:-1:1,binnm_nm(:,socd),'color',[147 91 161]./255) % Southern
plot(10:-1:1,binnm_nm(:,atld),'color',[36 170 226]./255) % Atlantic
plot(10:-1:1,binnm_nm(:,medd),'color',[9 129 54]./255) % Med
plot(10:-1:1,binnm_nm(:,arcd),'color',[221 11 21]./255) % Arc
plot(10:-1:1,binnm_nm(:,indd),'color',[201 110 170]./255) % Ind

axis([0.5 10.5 -3 3]);set(gca,'YTick',[-3:1:3]);grid on
set(gca,'XGrid','on')
set(gca,'YGrid','on')
set(gca,'XMinorGrid','on')
xticklabel=[' 100';' 300';' 500';' 700';' 900';'1100';'1300';'1500';'1700';'1900';];
set(gca,'XTickLabel',xticklabel)
boxplot(binnm_nm',floor(rangec),'widths',1,'Color',[0 0 0])
plot(10:-1:1,wavemn,'--','Linewidth',2,'Color',[0.5 0.5,0.5])
plot(10:-1:1,wavemd,'k','Linewidth',2)
axis([0.5 10.5 -3 3]);set(gca,'YTick',[-3:1:3]);grid on
set(gca,'XGrid','on')
set(gca,'YGrid','on')
set(gca,'XMinorGrid','on')
xlabel('Year (CE)')
ylabel({'Standardized SST anomaly';'(s.d. units)'})
title(['McGregor et al (2015), Fig 2a: Binned and standardized ' ...
       'reconstructed SST'])
hold off
grid on

print -dpdf composites_generated/Ocean2kLR2015sst_SF1a_F2a.pdf

close

% 4. Output the results for further digestion:

% 4.1. save compositing to ascii files (*.csv)

% Addison-mean bin composites: dimensions 10 bins x 57 sites
save -ascii composites_generated/stimem.csv stimem 
save -ascii composites_generated/binnm_nm.csv binnm_nm 
save -ascii composites_generated/binnNm.csv binnNm 

% 4.2. save the matlab workspace of all variables:
save composites_generated/compositework

